unit planet;

interface
uses Windows,GLScene, GLVectorFileObjects, GLObjects, GLCadencer,
  GLWin32Viewer, GLMisc, GLTexture, jpeg, Graphics, GLThorFX,GLFireFX,Geometry,VectorTypes,inifiles,sysutils,option;
type TAsteroid=class
private
  fmodel:TFreeForm;
  rolla,turna,pitcha,speed:Single;
  Sp:TVector4f;
public
  procedure Life;
  constructor Create(path:String;cadencer:TGLCadencer);
end;
type TMoon=class
private
  fmodel:TSphere;
  rotationspeed:Single;
  length:Single;
  espeed:Single;
  angle:Single;
  kx,ky,kz:Single;
public
  procedure Life;
  constructor Create(inif:TMemIniFile;num:Integer;planet:TSphere);
end;

type TPlanet=class
private
  fmodel:TSphere;
  rotationspeed:Single;
  Moons:Array[1..12] of TMoon;
public
  property Model:TSphere read fmodel write fmodel;
  constructor Create(path:String;cadencer:TGLCadencer);
  procedure Life;
end;

implementation
  constructor TPlanet.Create(path:String;cadencer:TGLCadencer);
  var inif:TMemIniFile; i:Integer;
  begin
    inif:=TMemIniFile.Create(path);
    rotationspeed:=inif.ReadFloat('General','rotationspeed',0);
    fmodel:=TSphere(cadencer.Scene.Objects.AddNewChild(TSphere));
    fmodel.Slices:=MainOption.Slice;
    fmodel.Stacks:=MainOption.Stack;
    fmodel.Material.Texture.Image.LoadFromFile('images\textures\planets\'+inif.ReadString('General','Texture',''));
    fmodel.Material.Texture.TextureMode:=tmModulate;
    fmodel.Material.Texture.Enabled:=True;
    fmodel.Radius:=inif.ReadFloat('General','radius',10);
    fmodel.Pitch(inif.ReadFloat('General','pitch',0));
    fmodel.Roll(inif.ReadFloat('General','roll',0));
    fmodel.Turn(inif.ReadFloat('General','turn',0));
    with fmodel.Position do begin
      x:=inif.ReadFloat('General','x',0);
      y:=inif.ReadFloat('General','y',0);
      z:=inif.ReadFloat('General','z',0);
    end;
    for i:=1 to inif.ReadInteger('General','mooncount',0) do begin
      Moons[i]:=TMoon.Create(inif,i,fmodel);
    end;
    inif.Destroy;
  end;
  procedure TPlanet.Life;
  var i:Byte;
  begin
    fmodel.Turn(rotationspeed);
    i:=1;
    while Assigned(Moons[i]) do begin
      Moons[i].Life;
      inc(i);
    end;
  end;

  //----------------------------------
  //----------------------------------

  constructor TMoon.Create(inif:TMemIniFile;num:Integer;planet:TSphere);
  var Sect:String;
  begin
    Sect:='Moon'+IntToStr(num);
    fmodel:=TSphere(planet.AddNewChild(TSphere));
    length:=inif.ReadFloat(Sect,'length',10);
    fmodel.Slide(length);
    rotationspeed:=inif.ReadFloat(Sect,'rotationspeed',0);
    fmodel.Slices:=MainOption.Slice;
    fmodel.Stacks:=MainOption.Stack;
    fmodel.Radius:=inif.ReadFloat(Sect,'radius',3);
    fmodel.Material.Texture.Image.LoadFromFile('images\textures\planets\'+inif.ReadString(Sect,'Texture',''));
    fmodel.Material.Texture.TextureMode:=tmModulate;
    fmodel.Material.Texture.Enabled:=True;
    fmodel.Pitch(inif.ReadFloat(Sect,'pitch',0));
    fmodel.Turn(inif.ReadFloat(Sect,'turn',0));
    fmodel.Roll(inif.ReadFloat(Sect,'roll',0));
    kx:=inif.ReadFloat(Sect,'kx',1);
    ky:=inif.ReadFloat(Sect,'ky',1);
    kz:=inif.ReadFloat(Sect,'kz',1);
    angle:=inif.ReadFloat(Sect,'startangle',0);
    espeed:=inif.ReadFloat(Sect,'speed',0);
  end;
  procedure TMoon.Life;
  begin
    fmodel.Turn(rotationspeed);
    angle:=angle+espeed;
    if angle>360 then angle:=angle-360;
    with fmodel.Position do begin
      X:=cos(DegToRad(angle))*length*kx;
      Y:=cos(DegToRad(angle))*length*ky;
      Z:=sin(DegToRad(angle))*length*kz;
    end;
  end;

  constructor TAsteroid.Create(path:String;cadencer:TGLCadencer);
  var inif:TMemIniFile;
  begin
    inif:=TMemIniFile.Create(path);
    fmodel:=TFreeForm(cadencer.Scene.Objects.AddNewChild(TFreeForm));
    fmodel.LoadFromFile('models/asteroids/'+inif.ReadString('General','Model',''));
    with fmodel.Material do begin
      Texture.Image.LoadFromFile('images/textures/asteroids/'+inif.ReadString('General','Texture',''));
      Texture.TextureMode:=tmModulate;
      Texture.Disabled:=False;
    end;
    fmodel.Scale.Scale(0.01);
    fmodel.Position.X:=inif.ReadFloat('General','X',0);
    fmodel.Position.Y:=inif.ReadFloat('General','Y',0);
    fmodel.Position.Z:=inif.ReadFloat('General','Z',0); Randomize;
    fmodel.PitchAngle:=inif.ReadFloat('General','StartPitch',random(360));
    fmodel.TurnAngle:=inif.ReadFloat('General','StartTurn',random(360));
    fmodel.RollAngle:=inif.ReadFloat('General','StartRoll',random(360));
    rolla:=inif.ReadFloat('General','RollSpeed',0);
    pitcha:=inif.ReadFloat('General','PitchSpeed',0);
    turna:=inif.ReadFloat('General','TurnSpeed',0);
    Sp:=fmodel.AbsoluteDirection;
    speed:=inif.ReadFloat('General','Speed',0);
    ScaleVector(Sp,speed);
  end;
  procedure TAsteroid.Life;
  begin
    fmodel.Roll(rolla);
    fmodel.Pitch(pitcha);
    fmodel.Turn(turna);
    fmodel.Translate(Sp[0],Sp[1],Sp[2]);
  end;
end.
